var express = require('express');
var path = require('path');
var favicon = require('serve-favicon');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
var domain = require('domain');
var cluster = require('cluster');

var app = express();

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'ejs');

// uncomment after placing your favicon in /public
//app.use(favicon(path.join(__dirname, 'public', 'favicon.ico')));
app.use(logger('dev'));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({
    extended: false
}));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public')));

// 加载扩展函数
require('./extensions/function_extend.js');
var Router = require('./framework/lib/Router.js');
var codeMap = require('./conf/code.inc.js');
require('./conf/config.inc.js');

var Response = require('./framework/lib/Response.js');
global['AppErrors'] = require('./framework/lib/AppErrors.js');

// catch 404 and forward to error handler
app.use(function(req, res, next) {
    var objDomain = domain.create();
    objDomain.run(function () {
        main(req, res, next);
    });

    objDomain.on('error', function (err) {
        var objResponse = new Response(req, res, codeMap);
        handlerException(objResponse, err);
    });
});

function handlerException(objResponse, err) {
    try {
        if (err instanceof AppErrors.Interrupt) {
            // console.log('catch Interrupt.');
        } else if (err instanceof AppErrors.DbError) {
            objResponse.error(CODE_DB_ERROR, null, err.message, null);
        } else if (err instanceof AppErrors.RedisError) {
            objResponse.error(CODE_REDIS_ERROR, null, err.message, null);
        } else {
            // 对于未知情况，则返回位置错误
            objResponse.error(CODE_UNKNOW_ERROT);
        }
    } catch(ex) {
        console.error('catch(ex)');
        console.error(ex);
    }

}

function main(req, res, next) {
    var objRouter = new Router(req.originalUrl);
    /**
     * @type {Doc}
     */
    if(req.query['doc']) {
        objRouter.genDoc(req.query['doc']);
    } else {
        /**
         * @type {Controller}
         */
        var SomeController = objRouter.getController();
        var objResponse = new Response(req, res, codeMap);

        if (SomeController) {
            var objController = new SomeController(req, res);
            var action = objRouter.getFullActionName();
            // 先获取get参数，然后获取post参数
            var args = req.query || req.body || {};
            if (objController[action]) {
                objController[action](args);
            } else {
                var msg = 'can not find function ' + objRouter.getActionName();
                objResponse.error(CODE_NOT_EXIST_INTERFACE, msg);
            }
        } else {
            var msg = 'can not find class ' + objRouter.getControllerName();
            objResponse.error(CODE_NOT_EXIST_INTERFACE, msg);
        }
    }
}

// 捕捉全局异常
// uncaughtException 避免程序崩溃
process.on('uncaughtException', function(err) {
    var CallLog = require('./framework/lib/CallLog.js');
    // 记录访问日志
    var objCallLog = new CallLog(null);
    var msg = err.message && err.message.message;
    objCallLog.logSelfCall(CODE_UNKNOW_ERROT, msg);
    setTimeout(function() {
        process.exit(1);
    }, 100);
});


module.exports = app;